// Mandelbrot shader using GPGPU techniques
// Author: Gabriel Zachmann, June 2007
//
// This shader computes nice colors from the iteration count
// stored in each texel using |renormalized iteration count'.


#version 110


uniform sampler2D zn;         // final texture containing z_n (or earlier z_i)

uniform float MaxIterations;    // max. num. of iterations
uniform vec4 InnerColor;      // two colors that the bands around (try: red)
uniform vec4 OuterColor;      // the mandelbrot set interpolate (try: blue)
uniform float BandFrequ;      // try: 0.1 .. 0.01


void main()
{
	vec4 inputValue = texture2D( zn, gl_TexCoord[0].xy );
	vec2 z = inputValue.xy;
	float n = inputValue.z;

	gl_FragColor = vec4( 0.13, 0.13, 0.13, 1.0 );	// = "inside" color
	if ( n < MaxIterations )
	{
		// classical coloring with "banding"
		//float f = n / MaxIterations;
		//gl_FragColor = mix( InnerColor, OuterColor, fract(f * BandFrequ) );

		// coloring using 'renormalized iteration count'
		// (see e.g. http://linas.org/art-gallery/escape/smooth.html )
		float f = n + 1.0 - log(log(length(z)));
		gl_FragColor = mix( InnerColor, OuterColor, f * BandFrequ );
	}

	gl_FragColor *= gl_Color;
}
