// Per-pixel lighting with a directional light

// the following variables are just to pass data from the vertex shader to the 
// fragment shader; they do not need to be interpolated; but that way we can balance
// the workload between the two shaders
varying vec4 diffuse, ambient;
varying vec3 lightDir, halfVector;

// this is the only variable that needs to be interpolated
varying vec3 normal;

void main()
{		
	// first transform the normal into eye space and normalize the result
	normal = normalize(gl_NormalMatrix * gl_Normal);

	// now normalize the light's direction. Note that according to the OpenGL
	// specification, the light is stored in eye space. Also since we're 
	// talking about a directional light, the position field is actually direction.
	lightDir = normalize( gl_LightSource[0].position.xyz );
	
	// the half-vector (assuming directional light)
#if 0
	// this half-vector is the same for all vertices
	halfVector = normalize( gl_LightSource[0].halfVector.xyz );
#else
	// this half-vector is per-vertex (better)
	halfVector = lightDir;
	vec4 view = gl_ModelViewMatrix * gl_Vertex;
	view.xyz /= view.w;
	halfVector -= view.xyz;						// h += (eye - view)
	halfVector = normalize( halfVector );
#endif

	// Compute the diffuse term
	diffuse = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse;

	// Compute the ambient and globalAmbient terms
	ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
	ambient += gl_LightModel.ambient * gl_FrontMaterial.ambient;
	
	gl_Position = ftransform();
}
